@@ -26,9 +26,9 @@ ll_lognormal <- function(object, withheld_y, withheld_mu) {
 dstudent <- function(x, df, mean, sd, ncp, log = FALSE) {
   # from metRology::dt.scaled()
   if (!log) {
-    return(stats::dt((x - mean)/sd, df, ncp = ncp, log = FALSE)/sd)
+    return(stats::dt((x - mean) / sd, df, ncp = ncp, log = FALSE) / sd)
   } else {
-    return(stats::dt((x - mean)/sd, df, ncp = ncp, log = TRUE) - log(sd))
+    return(stats::dt((x - mean) / sd, df, ncp = ncp, log = TRUE) - log(sd))
   }
 }
 
@@ -39,7 +39,7 @@ ll_student <- function(object, withheld_y, withheld_mu) {
 
 ll_nbinom1 <- function(object, withheld_y, withheld_mu) {
   phi <- exp(object$model$par[["ln_phi"]])
-  stats::dnbinom(x = withheld_y, size = withheld_mu/phi, mu = withheld_mu, log = TRUE)
+  stats::dnbinom(x = withheld_y, size = withheld_mu / phi, mu = withheld_mu, log = TRUE)
 }
 
 ll_nbinom2 <- function(object, withheld_y, withheld_mu) {
@@ -47,6 +47,7 @@ ll_nbinom2 <- function(object, withheld_y, withheld_mu) {
   stats::dnbinom(x = withheld_y, size = phi, mu = withheld_mu, log = TRUE)
 }
 
+# no longer used within sdmTMB_cv(); uses TMB report() instead
 ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
   family_func <- switch(object$family$family,
     gaussian = ll_gaussian,
@@ -57,7 +58,8 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
     Gamma = ll_gamma,
     nbinom1 = ll_nbinom1,
     nbinom2 = ll_nbinom2,
-    cli_abort(paste0(object$family$family, " not yet implemented. ",
+    cli_abort(paste0(
+      object$family$family, " not yet implemented. ",
       "Please file an issue on GitHub."
     ))
   )
@@ -66,10 +68,11 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
 
 #' Cross validation with sdmTMB models
 #'
-#' Facilitates cross validation with sdmTMB models. Returns log likelihood or
-#' expected log predictive density (ELPD) across left-out data. Has an option
-#' for leave-future-out cross validation. By default creates folds randomly but
-#' folds can be manually assigned.
+#' Facilitates cross validation with sdmTMB models. Returns the log likelihood
+#' of left-out data, which is similar in spirit to the ELPD (expected log
+#' pointwise predictive density). The function has an option for
+#' leave-future-out cross validation. By default, the function creates folds
+#' randomly but folds can be manually assigned via the `fold_ids` argument.
 #'
 #' @param formula Model formula.
 #' @param data A data frame.
@@ -106,13 +109,16 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
 #'           and CV log likelihood.
 #' * `models`: A list of models; one per fold.
 #' * `fold_loglik`: Sum of left-out log likelihoods per fold.
-#' * `fold_elpd`: Expected log predictive density per fold on left-out data.
 #' * `sum_loglik`: Sum of `fold_loglik` across all left-out data.
-#' * `elpd`: Expected log predictive density across all left-out data.
 #' * `pdHess`: Logical vector: Hessian was invertible each fold?
 #' * `converged`: Logical: all `pdHess` `TRUE`?
 #' * `max_gradients`: Max gradient per fold.
 #'
+#' Prior to \pkg{sdmTMB} version '0.3.0.9002', `elpd` was incorrectly returned as
+#' the log average likelihood, which is another metric you could compare models
+#' with, but not ELPD. For maximum likelihood, [ELPD is equivalent in spirit to
+#' the sum of the log likelihoods](https://github.com/pbs-assess/sdmTMB/issues/235).
+#'
 #' @details
 #' **Parallel processing**
 #'
@@ -143,7 +149,7 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
 #'
 #' See example below.
 #'
-#' @examplesIf inla_installed()
+#' @examples
 #' mesh <- make_mesh(pcod, c("X", "Y"), cutoff = 25)
 #'
 #' # Set parallel processing first if desired with the future package.
@@ -155,9 +161,6 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
 #'   family = tweedie(link = "log"), k_folds = 2
 #' )
 #'
-#' m_cv$fold_elpd
-#' m_cv$elpd
-#'
 #' m_cv$fold_loglik
 #' m_cv$sum_loglik
 #'
@@ -198,15 +201,16 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
 #' example_data <- m_lfocv$models[[1]]$data
 #' table(example_data$cv_fold, example_data$year)
 #' }
-sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
-  k_folds = 8, fold_ids = NULL,
-  lfo = FALSE,
-  lfo_forecast = 1,
-  lfo_validations = 5,
-  parallel = TRUE,
-  use_initial_fit = FALSE,
-  spde = deprecated(),
-  ...) {
+sdmTMB_cv <- function(
+    formula, data, mesh_args, mesh = NULL, time = NULL,
+    k_folds = 8, fold_ids = NULL,
+    lfo = FALSE,
+    lfo_forecast = 1,
+    lfo_validations = 5,
+    parallel = TRUE,
+    use_initial_fit = FALSE,
+    spde = deprecated(),
+    ...) {
   if (k_folds < 1) cli_abort("`k_folds` must be >= 1.")
 
   if (is_present(spde)) {
@@ -232,10 +236,11 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
       # Create lfo_validations + 1 folds, ordered sequentially
       data$cv_fold <- 1
       t_validate <- sort(unique(data[[time]]), decreasing = TRUE)
-      for (t in seq(1, (lfo_validations + lfo_forecast - 1))) {
+      for (t in seq(1, lfo_validations)) {
         # fold id increasing order + forecast
         data$cv_fold[data[[time]] == t_validate[t]] <- lfo_validations - t + 1 + lfo_forecast
       }
+      data$cv_fold <- as.numeric(as.factor(data$cv_fold))
     } else {
       dd <- lapply(split(data, data[[time]]), function(x) {
         x$cv_fold <- sample(rep(seq(1L, k_folds), nrow(x)), size = nrow(x))
@@ -275,8 +280,9 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
   }
   if ("offset" %in% names(dot_args)) {
     .offset <- eval(dot_args$offset)
-    if (parallel && !is.character(.offset) && !is.null(.offset))
+    if (parallel && !is.character(.offset) && !is.null(.offset)) {
       cli_abort("We recommend using a character value for 'offset' (indicating the column name) when applying parallel cross validation.")
+    }
   } else {
     .offset <- NULL
   }
@@ -307,8 +313,10 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
     }
     dot_args <- list(dot_args)[[1]]
     dot_args$offset <- NULL
-    .args <- c(list(data = dat_fit, formula = formula, time = time, mesh = mesh,
-      weights = weights, offset = .offset), dot_args)
+    .args <- c(list(
+      data = dat_fit, formula = formula, time = time, mesh = mesh,
+      weights = weights, offset = .offset
+    ), dot_args)
     fit1 <- do.call(sdmTMB, .args)
   }
 
@@ -340,10 +348,11 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
       dot_args$offset <- NULL
       args <- c(list(
         data = dat_fit, formula = formula, time = time, mesh = mesh, offset = .offset,
-        weights = weights, previous_fit = if (use_initial_fit) fit1 else NULL), dot_args)
+        weights = weights, previous_fit = if (use_initial_fit) fit1 else NULL
+      ), dot_args)
       object <- do.call(sdmTMB, args)
       # if (max(object$gradients) > 0.01) {
-        # object <- run_extra_optimization(object, nlminb_loops = 1L, newton_loops = 0L)
+      # object <- run_extra_optimization(object, nlminb_loops = 1L, newton_loops = 0L)
       # }
     }
 
@@ -353,6 +362,7 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
       cv_data <- data[data$cv_fold == k, , drop = FALSE]
     }
 
+    # FIXME: only use TMB report() below to be faster!
     # predict for withheld data:
     predicted <- predict(object, newdata = cv_data, type = "response")
     cv_data$cv_predicted <- predicted$est
@@ -360,26 +370,33 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
     withheld_y <- predicted[[response]]
     withheld_mu <- cv_data$cv_predicted
 
+    # FIXME: get LFO working with the TMB report() option below!
     # calculate log likelihood for each withheld observation:
+    # trickery to get the log likelihood of the withheld data directly
+    # from the TMB report():
+    if (!lfo) {
+      tmb_data <- object$tmb_data
+      tmb_data$weights_i <- ifelse(tmb_data$weights_i == 1, 0, 1) # reversed
+      new_tmb_obj <- TMB::MakeADFun(
+        data = tmb_data,
+        parameters = get_pars(object),
+        map = object$tmb_map,
+        random = object$tmb_random,
+        DLL = "sdmTMB",
+        silent = TRUE
+      )
+      lp <- object$tmb_obj$env$last.par.best
+      r <- new_tmb_obj$report(lp)
+      cv_loglik <- -1 * r$jnll_obs
+      cv_data$cv_loglik <- cv_loglik[tmb_data$weights_i == 1]
+    } else { # old method; doesn't work with delta models!
+      cv_data$cv_loglik <- ll_sdmTMB(object, withheld_y, withheld_mu)
+    }
 
-    # trickery to get the log likelihood of the withheld data directly from the TMB report():
-    # tmb_data <- object$tmb_data
-    # tmb_data$weights_i <- ifelse(tmb_data$weights_i == 1, 0, 1) # reversed
-    # new_tmb_obj <- TMB::MakeADFun(
-    #   data = tmb_data,
-    #   parameters = get_pars(object),
-    #   map = predicted_obj$fit_obj$tmb_map,
-    #   random = predicted_obj$fit_obj$tmb_random,
-    #   DLL = "sdmTMB",
-    #   silent = TRUE
-    # )
-    # lp <- object$tmb_obj$env$last.par.best
-    # r <- new_tmb_obj$report(lp)
-    # r$nll_obs
-    # cv_data$cv_loglik <- -1 * r$nll_obs
-
+    ## test
+    # x2 <- ll_sdmTMB(object, withheld_y, withheld_mu)
+    # identical(round(cv_data$cv_loglik, 6), round(x2, 6))
     # cv_data$cv_loglik <- ll_sdmTMB(object, withheld_y, withheld_mu)
-    cv_data$cv_loglik <- ll_sdmTMB(object, withheld_y, withheld_mu)
 
     list(
       data = cv_data,
@@ -391,8 +408,10 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
   }
 
   if (requireNamespace("future.apply", quietly = TRUE) && parallel) {
-    message("Running fits with `future.apply()`.\n",
-      "Set a parallel `future::plan()` to use parallel processing.")
+    message(
+      "Running fits with `future.apply()`.\n",
+      "Set a parallel `future::plan()` to use parallel processing."
+    )
     if (lfo) {
       out <- future.apply::future_lapply(seq_len(lfo_validations), fit_func, future.seed = TRUE)
     } else {
@@ -400,9 +419,11 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
     }
     # out <- lapply(seq_len(k_folds), fit_func)
   } else {
-    message("Running fits sequentially.\n",
+    message(
+      "Running fits sequentially.\n",
       "Install the future and future.apply packages,\n",
-      "set a parallel `future::plan()`, and set `parallel = TRUE` to use parallel processing.")
+      "set a parallel `future::plan()`, and set `parallel = TRUE` to use parallel processing."
+    )
     if (lfo) {
       out <- lapply(seq_len(lfo_validations), fit_func)
     } else {
@@ -413,28 +434,19 @@ sdmTMB_cv <- function(formula, data, mesh_args, mesh = NULL, time = NULL,
   models <- lapply(out, `[[`, "model")
   data <- lapply(out, `[[`, "data")
   fold_cv_ll <- vapply(data, function(.x) sum(.x$cv_loglik), FUN.VALUE = numeric(1L))
-  fold_cv_elpd <- vapply(data, function(.x)
-    log_sum_exp(.x$cv_loglik) - log(length(.x$cv_loglik)), FUN.VALUE = numeric(1L))
-  #fold_cv_ll <- vapply(data, function(.x) .x$cv_loglik[[1L]], FUN.VALUE = numeric(1L))
-  # fold_cv_ll_R <- vapply(data, function(.x) .x$cv_loglik_R[[1L]], FUN.VALUE = numeric(1L))
   data <- do.call(rbind, data)
   data <- data[order(data[["_sdm_order_"]]), , drop = FALSE]
   data[["_sdm_order_"]] <- NULL
   data[["_sdmTMB_time"]] <- NULL
   row.names(data) <- NULL
-  # bad_eig <- vapply(out, `[[`, "bad_eig", FUN.VALUE = logical(1L))
   pdHess <- vapply(out, `[[`, "pdHess", FUN.VALUE = logical(1L))
   max_grad <- vapply(out, `[[`, "max_gradient", FUN.VALUE = numeric(1L))
-  # converged <- all(!bad_eig) && all(pdHess)
   converged <- all(pdHess)
   list(
     data = data,
     models = models,
     fold_loglik = fold_cv_ll,
-    fold_elpd = fold_cv_elpd,
-    # fold_loglik_R = fold_cv_ll_R,
     sum_loglik = sum(data$cv_loglik),
-    elpd = log_sum_exp(data$cv_loglik) - log(length(data$cv_loglik)),
     converged = converged,
     pdHess = pdHess,
     max_gradients = max_grad
